@@ -4,8 +4,10 @@
 #pragma once
 
 #include "glaze/beve/header.hpp"
+#include "glaze/beve/lazy.hpp"
 #include "glaze/beve/ptr.hpp"
 #include "glaze/beve/read.hpp"
+#include "glaze/beve/size.hpp"
 #include "glaze/beve/wrappers.hpp"
 #include "glaze/beve/write.hpp"
 #include "glaze/core/as_array_wrapper.hpp"
