@@ -24,6 +24,7 @@ namespace glz
    inline constexpr uint32_t MSGPACK = 30;
    inline constexpr uint32_t NDJSON = 100; // new line delimited JSON
    inline constexpr uint32_t TOML = 400;
+   inline constexpr uint32_t YAML = 450;
    inline constexpr uint32_t STENCIL = 500;
    inline constexpr uint32_t MUSTACHE = 501;
    inline constexpr uint32_t CSV = 10000;
@@ -387,6 +388,16 @@ namespace glz
       }
    }
 
+   consteval bool check_null_terminated(auto&& Opts)
+   {
+      if constexpr (requires { Opts.null_terminated; }) {
+         return Opts.null_terminated;
+      }
+      else {
+         return false;
+      }
+   }
+
    consteval bool check_partial_read(auto&& Opts)
    {
       if constexpr (requires { Opts.partial_read; }) {
@@ -1322,6 +1333,14 @@ namespace glz
       ret.format = TOML;
       return ret;
    }
+
+   template <auto Opts>
+   constexpr auto set_yaml()
+   {
+      auto ret = Opts;
+      ret.format = YAML;
+      return ret;
+   }
 }
 
 namespace glz
