@@ -274,8 +274,10 @@ namespace glz
             return;
          }
          if (!value) {
-            if constexpr (check_allocate_raw_pointers(Opts)) {
-               value = new std::remove_pointer_t<std::remove_cvref_t<Value>>{};
+            if constexpr (can_allocate_raw_pointer<Opts, std::decay_t<Ctx>>) {
+               if (!try_allocate_raw_pointer<Opts>(value, ctx)) {
+                  return;
+               }
             }
             else {
                ctx.error = error_code::invalid_nullable_read;
@@ -557,7 +559,7 @@ namespace glz
       template <auto Opts, class Value, is_context Ctx, class It, class End>
       GLZ_ALWAYS_INLINE static void op(Value&& value, uint8_t tag, Ctx&& ctx, It& it, const End& end) noexcept
       {
-         if constexpr (Opts.structs_as_arrays) {
+         if constexpr (check_structs_as_arrays(Opts)) {
             size_t len{};
             if (!msgpack::read_array_length(ctx, tag, it, end, len)) {
                return;
