@@ -708,6 +708,10 @@ namespace glz
       }
    };
 
+#ifdef _MSC_VER
+#pragma warning(push)
+#pragma warning(disable : 4702) // unreachable code from if constexpr
+#endif
    template <writable_array_t T>
    struct to<MSGPACK, T>
    {
@@ -740,6 +744,9 @@ namespace glz
             }
          }
       }
+#ifdef _MSC_VER
+#pragma warning(pop)
+#endif
    };
 
    template <>
