@@ -35,10 +35,6 @@
 #include "glaze/net/http_router.hpp"
 #include "glaze/util/parse.hpp"
 
-#ifdef GLZ_ENABLE_SSL
-#include <asio/ssl.hpp>
-#endif
-
 namespace glz
 {
    // WebSocket opcode constants
@@ -157,8 +153,8 @@ namespace glz
             size_t i = 0;
             size_t j = (context->count[0] >> 3) & 63;
 
-            if ((context->count[0] += uint32_t(len << 3)) < (len << 3)) context->count[1]++;
-            context->count[1] += (len >> 29);
+            if ((context->count[0] += uint32_t(len << 3)) < uint32_t(len << 3)) context->count[1]++;
+            context->count[1] += uint32_t(len >> 29);
 
             if ((j + len) > 63) {
                std::memcpy(&context->buffer[j], data, (i = 64 - j));
